
/*
 * Copyright (C) JOSH Development Team
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 2 for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Please visit www.joshvm.org if you need additional information or
 * have any questions.
 */
import java.util.Random;

import org.joshvm.ui.Color;
import org.joshvm.ui.Display;
import org.joshvm.ui.Screen;
import org.joshvm.ui.UIObject;
import org.joshvm.ui.listener.ClickedListener;
import org.joshvm.ui.widget.Button;
import org.joshvm.ui.widget.Canvas;

public class UIDemoA {
    private static final int TYPE_LINE = 0;
    private static final int TYPE_ARC = 1;
    private static final int TYPE_RECT = 2;
    private static final int TYPE_MAX = 3;

    private static final int canvasW = 400;
    private static final int canvasH = 400;

    private Display display = Display.getDisplay();
    private Canvas canvas;
    private boolean stopped;

    private int type = TYPE_LINE;
    private Random random = new Random();

    private Color getRandomColor() {
        Color color = new Color();
        color.setRed(random.nextInt(256));
        color.setGreen(random.nextInt(256));
        color.setBlue(random.nextInt(256));
        return color;
    }

    private int getRandomX() {
        return 20 + random.nextInt(canvasW - 40);
    }

    private int getRandomY() {
        return 20 + random.nextInt(canvasH - 40);
    }

    private void drawLine() {
        int x1 = getRandomX();
        int x2 = getRandomX();
        int y1 = getRandomY();
        int y2 = getRandomY();

        while (Math.abs(x1 - x2) + Math.abs(y1 - y2) < 10) {
            x2 = x1 + 10;
        }

        canvas.setLineColorOfDraw(getRandomColor());
        canvas.setLineWidthOfDraw(1 + random.nextInt(5));
        canvas.drawLine(x1, y1, x2, y2);
    }

    private void drawCircle() {
        int x1 = getRandomX();
        int y1 = getRandomY();

        int r = 0;
        do {
            r = random.nextInt(canvasW);
        } while (r > x1 || r > canvasW - x1 || r > y1
                || r > canvasH - y1);

        if (r == 0) {
            r = 4;
        }
        canvas.setLineColorOfDraw(getRandomColor());
        canvas.setLineWidthOfDraw(r);
        canvas.drawArc(x1, y1, r, 0, 360);
    }

    private void drawRect() {
        int x1 = getRandomX();
        int x2 = getRandomX();
        int y1 = getRandomY();
        int y2 = getRandomY();

        int w = x2 - x1;
        if (w < 0) {
            w = -w;
            x1 = x2;
        }
        if (w == 0) {
            w = 2;
        }
        int h = y2 - y1;
        if (h < 0) {
            h = -h;
            y1 = y2;
        }
        if (h == 0) {
            h = 2;
        }
        canvas.setBackgroundColorOfDraw(getRandomColor());
        canvas.setLineColorOfDraw(getRandomColor());
        canvas.setLineWidthOfDraw(1 + random.nextInt(5));
        canvas.setRectRadiusOfDraw(random.nextInt(5));
        canvas.setShadowColorOfDraw(getRandomColor());
        canvas.setShawdowWidthOfDraw(random.nextInt(5));
        canvas.setShadowOffsetXOfDraw(random.nextInt(5));
        canvas.setShadowOffsetYOfDraw(random.nextInt(5));
        canvas.drawRect(x1, y1, w, h);
    }

    private void cleanCanvas() {
        canvas.fill(new Color(0));
    }

    private void draw() {
        cleanCanvas();

        type = random.nextInt(TYPE_MAX);
        switch (type) {
        case TYPE_LINE:
            drawLine();
            break;
        case TYPE_ARC:
            drawCircle();
            break;
        case TYPE_RECT:
            drawRect();
            break;
        default:
            break;
        }
    }

    private void run() {
        Screen screen = new Screen();
        canvas = new Canvas(canvasW, canvasH);
        canvas.setAlign(UIObject.ALIGN_CENTER, UIObject.ALIGN_CENTER);
        Button btn = new Button();
        btn.setText("next");
        btn.setClickedListener(new ClickedListener() {

            public void onClicked(UIObject target) {
                draw();
            }

        });
        btn.setAlign(UIObject.ALIGN_RIGHT, UIObject.ALIGN_CENTER);
        btn.setX(-50);
        screen.addObject(btn);
        screen.addObject(canvas);
        new Thread() {
            public void run() {
                while (!stopped) {
                    draw();
                    try {
                        Thread.sleep(2000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        }.start();
        display.setScreen(screen);
    }

    private void stop() {
        stopped = true;
        display.stop();
    }

    public static void main(String[] args) {
        UIDemoA demo = new UIDemoA();
        System.out.println("===start===");
        demo.run();
        try {
            Thread.sleep(60000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        System.out.println("===stop===");
        demo.stop();
    }
}
